<?php

/*
Plugin Name: OxaPay Payment Method for Paid Memberships Pro
Plugin URI: https://www.oxapay.com
Description: OxaPay Payment Method for Paid Memberships Pro
Version: 1.0
Author: OxaPay
Author URI: https://www.oxapay.com
Text Domain: oxapay-payment-for-pmmp
 */

if ( ! defined( 'ABSPATH' ) ) exit;

add_action('plugins_loaded', function() {
	load_plugin_textdomain('oxapay-payment-for-pmmp', false, basename(dirname(__FILE__)) . '/languages');

	class PMProGateway_oxapay extends PMProGateway
	{
		function __construct($gateway = NULL)
		{
			$this->gateway = __('OxaPay Payment Method for Paid Memberships Pro', 'oxapay-payment-for-pmmp');
			$this->gateway = $gateway;
			return $this->gateway;
		}										
	
		static function init()
		{
			add_filter('pmpro_gateways', array('PMProGateway_oxapay', 'pmpro_gateways'));
			add_filter('pmpro_payment_options', array('PMProGateway_oxapay', 'pmpro_payment_options'));
			add_filter('pmpro_payment_option_fields', array('PMProGateway_oxapay', 'pmpro_payment_option_fields'), 10, 2);
			if (pmpro_getOption("gateway") == "oxapay")
			{
				add_filter('pmpro_include_billing_address_fields', '__return_false');
				add_filter('pmpro_include_payment_information_fields', '__return_false');
				add_filter('pmpro_required_billing_fields', array('PMProGateway_oxapay', 'pmpro_required_billing_fields'));
				add_filter('pmpro_checkout_before_change_membership_level', array('PMProGateway_oxapay', 'pmpro_checkout_before_change_membership_level'), 10, 2);
			}
			add_action('wp_ajax_oxapay', array('PMProGateway_oxapay', 'verifyOxaPay'));
			add_action('wp_ajax_nopriv_oxapay', array('PMProGateway_oxapay', 'verifyOxaPay'));
		}
	
		static function pmpro_gateways($gateways)
		{
			if(empty($gateways['oxapay']))
				$gateways['oxapay'] = __('OxaPay', 'oxapay-payment-for-pmmp');
			return $gateways;
		}
		static function getGatewayOptions()
		{
			$options = array('oxapay_merchant','oxapay_currency','oxapay_lifetime');
			return $options;
		}
		static function pmpro_payment_options($options)
		{
			$oxapay_options = PMProGateway_oxapay::getGatewayOptions();
			$options = array_merge($oxapay_options, $options);
			return $options;
		}
		static function pmpro_required_billing_fields($fields)
		{
			unset($fields['bfirstname']);
			unset($fields['blastname']);
			unset($fields['baddress1']);
			unset($fields['bcity']);
			unset($fields['bstate']);
			unset($fields['bzipcode']);
			unset($fields['bphone']);
			unset($fields['bemail']);
			unset($fields['bcountry']);
			unset($fields['CardType']);
			unset($fields['AccountNumber']);
			unset($fields['ExpirationMonth']);
			unset($fields['ExpirationYear']);
			unset($fields['CVV']);
			return $fields;
		}
		static function pmpro_payment_option_fields($values, $gateway)
		{
			$currencies = array('' => __('Select By User', 'oxapay-payment-for-pmmp'));
			$h = array('Content-Type' => 'application/json', 'Accept' => 'application/json');
			$result = wp_remote_post($u = 'https://api.oxapay.com/merchants/allowedCoins', $a = array('timeout' => 25, 'body' => wp_json_encode($b = array('merchant' => $values['oxapay_merchant'])), 'headers' => $h, 'sslverify' => false));
			if (!is_wp_error($result))
			{
				$coins = json_decode(wp_remote_retrieve_body($result));
				if ($coins->result == 100)
				{
					foreach ($coins->allowed as $coin) {
						$currencies[$coin] = __($coin, 'oxapay-payment-for-pmmp');
					}
				}
			}
			?>
			<tr class="pmpro_settings_divider gateway gateway_oxapay" <?php if($gateway != "oxapay") { ?>style="display: none;"<?php } ?>>
				<td colspan="2">
					<?php _e('OxaPay Settings', 'oxapay-payment-for-pmmp'); ?>
				</td>
			</tr>
			<tr class="gateway gateway_oxapay" <?php if($gateway != "oxapay") { ?>style="display: none;"<?php } ?>>
				<th scope="row" valign="top">
					<label for="oxapay_merchant"><?php _e('OxaPay Merchant Key', 'oxapay-payment-for-pmmp'); ?>:</label>
				</th>
				<td>
					<input type="text" id="oxapay_merchant" name="oxapay_merchant" size="50" value="<?php echo esc_attr($values['oxapay_merchant'])?>" />
				</td>
			</tr>
			<tr class="gateway gateway_oxapay" <?php if($gateway != "oxapay") { ?>style="display: none;"<?php } ?>>
				<th scope="row" valign="top">
					<label for="oxapay_currency"><?php _e('Currency (optional)', 'oxapay-payment-for-pmmp'); ?>:</label>
				</th>
				<td>
					<select name="oxapay_currency" id="oxapay_currency">
						<?php foreach ($currencies as $key => $value) { ?>
						<option <?php if($values['oxapay_currency'] == $key) { echo 'selected';} ?> value="<?php echo $key; ?>"><?php echo $value; ?></option>
						<?php } ?>
					</select>
				</td>
			</tr>
			<tr class="gateway gateway_oxapay" <?php if($gateway != "oxapay") { ?>style="display: none;"<?php } ?>>
				<th scope="row" valign="top">
					<label for="oxapay_lifetime"><?php _e('Payment Lifetime In Minutes (15-2880)', 'oxapay-payment-for-pmmp'); ?>:</label>
				</th>
				<td>
					<input type="number"  min="15" max="2880" id="oxapay_lifetime" name="oxapay_lifetime" size="50" value="<?php echo esc_attr($values['oxapay_lifetime']?:60)?>" />
				</td>
			</tr>
			<?php
		}

		static function pmpro_checkout_before_change_membership_level($user_id, $morder)
		{
			global $discount_code_id, $wpdb;
			if (empty($morder)) {
				return;
			}
			$morder->user_id = $user_id;
			$morder->saveOrder();
			if (!empty($discount_code_id)) {
				$wpdb->query("INSERT INTO $wpdb->pmpro_discount_codes_uses (code_id, user_id, order_id, timestamp) VALUES('" . $discount_code_id . "', '" . $user_id . "', '" . $morder->id . "', now())");	
			}
			$morder->Gateway->sendToOxaPay($morder);
		}
		function process(&$order)
		{
			$now = time();
			if(pmpro_isLevelRecurring($order->membership_level))
			{
				_e('OxaPay Not Support Recurring Payments', 'oxapay-payment-for-pmmp');
				exit;
			}
			$initial_payment = $order->InitialPayment;
			$initial_payment_tax = $order->getTaxForPrice($initial_payment);
			$initial_payment = round((float)$initial_payment + (float)$initial_payment_tax, 2);
			$b = array(
				'merchant'    => pmpro_getOption("oxapay_merchant"),
				'lifeTime'    => pmpro_getOption("oxapay_lifetime"),
				'amount'      => floatval($initial_payment),
				'description' => $order->membership_level->name . " at " . get_bloginfo("name"),
				'orderId'     => strval($now),
				'email'       => $order->Email,
				'callbackUrl' => admin_url("admin-ajax.php") . "?action=oxapay",
				'returnUrl'   => pmpro_url("confirmation", "?level=" . $order->membership_level->id),
			);
			if (pmpro_getOption("oxapay_currency")) {
				$b['currency'] = pmpro_getOption("oxapay_currency");
			}
			$h = array('Content-Type' => 'application/json', 'Accept' => 'application/json');
			$result = wp_remote_post($u = 'https://api.oxapay.com/merchants/request', $a = array('timeout' => 25, 'body' => wp_json_encode($b), 'headers' => $h, 'sslverify' => false));
			if (is_wp_error($result))
			{
				$message = __('Erorr', 'oxapay-payment-for-pmmp') . ' : ' . $result->get_error_message();
				$order->error = $message;
				return false;
			}
			$pay = json_decode(wp_remote_retrieve_body($result));
			if ($pay->result == 100)
			{
				$_SESSION['oxapay_trackid'] = $pay->trackId;
				$order->code = $now;
				$order->payment_type = "OxaPay";
				$order->status = "review";
				$order->saveOrder();
				return true;
			}
			else
			{
				$message = __('Erorr', 'oxapay-payment-for-pmmp') . ' : ' . $pay->message;
				$order->error = $message;
				return false;
			}
		}
		function sendToOxaPay(&$order)
		{
			$payLink = 'https://oxapay.com/mpay/' . $_SESSION['oxapay_trackid']; // $order->code
			echo '<p>'.__('thank you for your order. you are redirecting to oxapay gateway. please wait', 'oxapay-payment-for-pmmp').'</p>';
			echo '<a href="'.esc_url($payLink).'" class="oxapay_btn btn button">'.__('Pay', 'oxapay-payment-for-pmmp').'</a>';
			echo '<script>document.location = "'.esc_url($payLink).'";</script>';
			exit;
		}
		function verifyOxaPay()
		{
			$postData = file_get_contents('php://input');
			$data = json_decode($postData);
			if ($data->type === 'payment') {
				if ($_SERVER['HTTP_HMAC'] === hash_hmac('sha512', $postData, pmpro_getOption("oxapay_merchant"))) {
					$morder = new MemberOrder();
					$morder->getMemberOrderByCode($data->orderId);

					if ($data->status == 'Waiting') {
						$message = __('Payment Status is Waiting', 'oxapay-payment-for-pmmp');
						$morder->error = $message;
						$morder->saveOrder();
					} elseif ($data->status == 'Failed') {
						$message = __('Payment Status is Failed', 'oxapay-payment-for-pmmp');
						$morder->error = $message;
						$morder->saveOrder();
					} elseif ($data->status == 'Expired') {
						$message = __('Payment Status is Expired', 'oxapay-payment-for-pmmp');
						$morder->error = $message;
						$morder->saveOrder();
					} elseif ($data->status != 'Paid') {
						$message = sprintf(__('Payment Status is Not Paid (%s)', 'oxapay-payment-for-pmmp'), $data->status);
						$morder->error = $message;
						$morder->saveOrder();
					} else {
						if (PMProGateway_oxapay::pmpro_ipnChangeMembershipLevel($data->orderId, $morder)) {
							// nothing to do ?
						} else {
							$message = sprintf(__("ERROR: Couldn't change level for order (%s)", 'oxapay-payment-for-pmmp'), $morder->code);
							$morder->error = $message;
							$morder->saveOrder();
						}
					}
					http_response_code(200);
					echo 'ok';
					exit;
				} else {
					http_response_code(400);
					echo 'Invalid HMAC signature';
					exit;
				}
			} else {
				http_response_code(400);
				echo 'Invalid data.type';
				exit;
			}
		}
		static function pmpro_ipnChangeMembershipLevel($txn_id, &$morder)
		{
			$morder->membership_level = apply_filters("pmpro_ipnhandler_level", $morder->getMembershipLevel(), $morder->user_id);
			if(!empty($morder->membership_level->expiration_number))
			{
				$enddate = "'" . date("Y-m-d", strtotime("+ " . $morder->membership_level->expiration_number . " " . $morder->membership_level->expiration_period, current_time("timestamp"))) . "'";
			}
			else
			{
				$enddate = "NULL";
			}
			$morder->getDiscountCode();
			if(!empty($morder->discount_code))
			{		
				$morder->getMembershipLevel(true);
				$discount_code_id = $morder->discount_code->id;
			}
			else
			{
				$discount_code_id = "";
			}
			$startdate = apply_filters("pmpro_checkout_start_date", "'" . current_time('mysql') . "'", $morder->user_id, $morder->membership_level);
			$custom_level = array(
				'user_id' => $morder->user_id,
				'membership_id' => $morder->membership_level->id,
				'code_id' => $discount_code_id,
				'initial_payment' => $morder->membership_level->initial_payment,
				'billing_amount' => $morder->membership_level->billing_amount,
				'cycle_number' => $morder->membership_level->cycle_number,
				'cycle_period' => $morder->membership_level->cycle_period,
				'billing_limit' => $morder->membership_level->billing_limit,
				'trial_amount' => $morder->membership_level->trial_amount,
				'trial_limit' => $morder->membership_level->trial_limit,
				'startdate' => $startdate,
				'enddate' => $enddate
			);
			if(pmpro_changeMembershipLevel($custom_level, $morder->user_id) !== false)
			{
				$morder->status = "success";
				$morder->payment_transaction_id = $txn_id;
				$morder->subscription_transaction_id = "";
				$morder->saveOrder();
				if(!empty($discount_code) && !empty($use_discount_code))
				{
					$wpdb->query("INSERT INTO $wpdb->pmpro_discount_codes_uses (code_id, user_id, order_id, timestamp) VALUES('" . $discount_code_id . "', '" . $morder->user_id . "', '" . $morder->id . "', '" . current_time('mysql') . "");
				}
				$invoice = new MemberOrder($morder->id);
				$user = get_userdata($morder->user_id);
				$user->membership_level = $morder->membership_level;
				$pmproemail = new PMProEmail();
				$pmproemail->sendCheckoutEmail($user, $invoice);
				$pmproemail = new PMProEmail();
				$pmproemail->sendCheckoutAdminEmail($user, $invoice);
				return true;
			} else {
				return false;
			}
		}
	}
	add_filter('plugin_action_links_' . plugin_basename(__FILE__), function($links) {
		return array_merge(array('settings' => '<a href="'.admin_url('admin.php?page=pmpro-paymentsettings').'">'.__('Settings', 'oxapay-payment-for-pmmp').'</a>'), $links);
	});
	add_action('init', array('PMProGateway_oxapay', 'init'));
}, 666);

?>